@@ -6,16 +6,16 @@ use Carp qw/croak/;
 use POSIX ":sys_wait_h";
 use List::Util qw/first/;
 use Time::HiRes qw/sleep/;
+use Fcntl qw/LOCK_EX LOCK_UN LOCK_NB/;
 
 use File::Spec();
 
 use Test2::Harness::Util qw/write_file_atomic/;
 
-use Test2::Harness::Run::Runner::ProcMan::Locker();
 use Test2::Harness::Util::File::JSONL();
 use Test2::Harness::Run::Queue();
 
-our $VERSION = '0.001041';
+our $VERSION = '0.001042';
 
 use Test2::Harness::Util::HashBase qw{
     -pid
@@ -23,7 +23,9 @@ use Test2::Harness::Util::HashBase qw{
     -jobs   -jobs_file -jobs_seen
     -stages
 
-    -locker
+    -slots
+
+    -_lock -lock_file
     -pending
     -_pids
     -end_loop_cb
@@ -66,11 +68,6 @@ sub init {
     $self->{+JOBS} ||= Test2::Harness::Util::File::JSONL->new(name => $self->{+JOBS_FILE});
     $self->{+JOBS_SEEN} = {};
 
-    $self->{+LOCKER} = Test2::Harness::Run::Runner::ProcMan::Locker->new(
-        dir => $self->{+DIR},
-        slots => $self->{+RUN}->job_count,
-    );
-
     $self->read_jobs();
     $self->preload_queue();
 }
@@ -191,6 +188,8 @@ sub wait_on_jobs {
 
         my $params = delete $self->{+_PIDS}->{$pid};
         my $cat = $params->{task}->{category};
+        $cat = 'long' if $cat eq 'medium';
+        $self->{+SLOTS}->{$cat}++;
 
         unless ($check == $pid) {
             $exit = -1;
@@ -199,6 +198,8 @@ sub wait_on_jobs {
 
         $self->write_exit(%$params, exit => $exit);
     }
+
+    $self->unlock unless keys %{$self->{+_PIDS}};
 }
 
 sub write_remaining_exits {
@@ -217,42 +218,107 @@ sub next {
     my $self = shift;
     my ($stage) = @_;
 
-    $self->poll_tasks;
-    my $list = $self->{+PENDING}->{$stage} or return;
+    my $task = $self->_next($stage);
 
-    my $wait_time = $self->{+WAIT_TIME};
-    my $end_cb = $self->{+END_LOOP_CB};
-
-    my $gen = 0;
-    my $no_gen;
-    while (@$list || !$self->{+QUEUE_ENDED}) {
-        return if $end_cb && $end_cb->();
-        $no_gen = 0 if $self->poll_tasks;
-        $self->wait_on_jobs;
-
-        my %tried;
-        for (my $i = 0; $i < @$list; $i++) {
-            my $task = $list->[$i];
-
-            my $cat = $task->{category};
-            $gen++ if $cat eq 'general';
-            next if $tried{$cat}++;
-
-            my $meth = "get_$cat";
-            $meth = "get_general" if $no_gen && ($cat eq 'long' || $cat eq 'medium');
-
-            my $lock = $self->{+LOCKER}->$meth or next;
-
-            splice(@$list, $i, 1);
-
-            return ($task, $lock);
-        }
-
-        $no_gen = !$gen;
-        sleep($wait_time) if $wait_time;
+    unless ($task) {
+        $self->wait_on_jobs() while keys %{$self->{+_PIDS}};
     }
 
-    return;
+    return $task;
+}
+
+sub lock {
+    my $self = shift;
+    return 1 if $self->{+_LOCK};
+    return 1 unless $self->{+LOCK_FILE};
+
+    open(my $lock, '>>', $self->{+LOCK_FILE}) or die "Could not open lock file: $!";
+    flock($lock, LOCK_EX | LOCK_NB) or next;
+    $self->{+_LOCK} = $lock;
+
+    return 1;
+}
+
+sub unlock {
+    my $self = shift;
+
+    my $lock = delete $self->{+_LOCK} or return 1;
+    flock($lock, LOCK_UN);
+    close($lock);
+    return 1;
+}
+
+sub _next {
+    my $self = shift;
+    my ($stage) = @_;
+
+    my $end_cb    = $self->{+END_LOOP_CB};
+    my $list      = $self->{+PENDING}->{$stage} ||= [];
+    my $wait_time = $self->{+WAIT_TIME};
+
+    my $max = $self->{+RUN}->job_count || 1;
+    my $slow = $max - 1;
+
+    my $slots = $self->{+SLOTS} ||= {
+        immiscible => 1,
+        isolation  => 1,
+        long       => $max - 1,
+        general    => $max,
+    };
+
+    my $first = 1;
+    while (@$list || !$self->{+QUEUE_ENDED}) {
+        return if $end_cb && $end_cb->();
+        sleep $wait_time unless $first;
+        $first = 0;
+
+        $self->poll_tasks;
+        $self->wait_on_jobs;
+
+        next unless @$list;
+        next unless $self->lock;
+
+        my $running = 0;
+        my %cats;
+        for my $job (values %{$self->{+_PIDS}}) {
+            $running++;
+            $cats{$job->{task}->{category}}++;
+        }
+
+        # At max jobs already
+        next if $running >= $max;
+
+        # None if isolation is running
+        next if $cats{isolation};
+
+        return shift @$list if $max == 1;
+
+        my $fallback;
+        for(my $i = 0; $i < @$list; $i++) {
+            my $cat = $list->[$i]->{category};
+            $cat = 'long' if $cat eq 'medium';
+
+            die "Unknown category: $cat" unless defined $slots->{$cat};
+
+            $fallback = $i if $cat eq 'long';
+
+            next unless $slots->{$cat};
+            $self->{+SLOTS}->{$cat}--;
+            return scalar splice(@$list, $i, 1);
+        }
+
+        # If we have a long one, but no generals, we can go ahead and run the long one
+        return scalar splice(@$list, $fallback, 1)
+            if $fallback;
+    }
 }
 
 1;
+
+__END__
+    long       => 1,
+    medium     => 1,
+    general    => 1,
+    isolation  => 1,
+    immiscible => 1,
+
